<?php
/**
 * Обязательный файл с описанием модуля, содержащий инсталлятор/деинсталлятор модуля.
 */

// Путь до папки нашего модуля
define('PATH_TO_MODULE_DELOBANK', realpath(__DIR__ . '/../'));

// Пролог модуля - подключение языковых файлов
require_once (PATH_TO_MODULE_DELOBANK . '/prolog.php');

/**
 * Класс для инсталляции и деинсталляции модуля
 */
class Delobank extends CModule
{
    const MODULE_ID = 'delobank';
    /** @var string Имя партнера - автора модуля. */
    public $PARTNER_NAME = 'delobank';
    /** @var string URL партнера - автора модуля. */
    public $PARTNER_URI = 'https://delo.ru/';
    /** @var string Версия модуля. */
    public $MODULE_VERSION;
    /** @var string Дата и время создания модуля. */
    public $MODULE_VERSION_DATE;
    /** @var string Имя модуля. */
    public $MODULE_NAME;
    /** @var string Описание модуля. */
    public $MODULE_DESCRIPTION;
    /** @var string ID модуля */
    public $MODULE_ID;
    /** @var int сортировка модуля */
    public $MODULE_SORT = 0;
    /** @var string массив с ошибками для обмена между методами */
    private $errors = '';

    public function __construct()
    {
        $this->MODULE_ID = self::MODULE_ID;
        $this->MODULE_NAME = GetMessage(DELOBANK_LANG_PREFIX . 'INSTALL_NAME');
        $this->MODULE_DESCRIPTION = GetMessage(DELOBANK_LANG_PREFIX . 'INSTALL_DESCRIPTION');
        $arModuleVersion = [];
        include PATH_TO_MODULE_DELOBANK . '/install/version.php';
        $this->MODULE_VERSION = $arModuleVersion['VERSION'];
        $this->MODULE_VERSION_DATE = $arModuleVersion['VERSION_DATE'];
    }

    public function DoInstall()
    {
        global $APPLICATION, $DB, $errors;
        // Создаёт таблицу в БД.
        $DB->RunSQLBatch(PATH_TO_MODULE_DELOBANK . '/install/db/mysql/install.sql');
        // Копирует нужные файлы в нужные места.
        if (!CModule::IncludeModule($this->MODULE_ID)) {
            if (!$this->validateRequirements()) {
                $this->inLog($this->errors);
                $errors = $this->errors;
            } else {
                if ($this->InstallFiles()) {
                    RegisterModule($this->MODULE_ID);
                } else {
                    $this->inLog($this->errors);
                    $errors = $this->errors;
                }
            }
        } else {
            $this->errors = 'Модуль уже установлен!';
            $this->inLog($this->errors);
            $errors = $this->errors;
        }
        // Показывает страницу с результатом установки модуля.
        $APPLICATION->IncludeAdminFile(GetMessage(DELOBANK_LANG_PREFIX . 'INSTALL_NAME'), PATH_TO_MODULE_DELOBANK . '/install/step_ok.php');
    }

    public function DoUninstall()
    {
        global $APPLICATION, $DB;
        if (isset($_REQUEST['uninstall']) && $_REQUEST['uninstall'] == 'Y' && CModule::IncludeModule($this->MODULE_ID)) {
            if ($this->UnInstallFiles()) {
                UnRegisterModule($this->MODULE_ID);
                $DB->RunSQLBatch(PATH_TO_MODULE_DELOBANK . '/install/db/mysql/uninstall.sql');
            }
        } else {
            // Показывает страницу с настройками удаления модуля.
            $APPLICATION->IncludeAdminFile(GetMessage(DELOBANK_LANG_PREFIX . 'INSTALL_NAME'), PATH_TO_MODULE_DELOBANK . '/install/unstep_ok.php');
        }
    }

    /**
     * @return bool|void
     */
    public function InstallFiles()
    {
        $path_from = PATH_TO_MODULE_DELOBANK . '/install/www';
        if (!CopyDirFiles($path_from, $_SERVER['DOCUMENT_ROOT'], true, true)) {
            $this->errors = GetMessage(DELOBANK_LANG_PREFIX . 'INSTALL_ERROR');
            return false;
        }
        // Сделаем подключение в init.php нашего файла
        $pathInitPhp = $this->getPathInitPhp();
        $delobankInit = $this->includeCodeDelobankInitPhp();
        if (file_exists($pathInitPhp)) {
            $contentInitPhp = file_get_contents($pathInitPhp);
            $pos = strrpos($contentInitPhp, self::MODULE_ID . '_init.php');
            if ($pos !== false) {
                // что-то пошло не так, уже было добавлено подключение наших файлов
                $error = 'В файле init.php уже есть подключение нашего файла';
                $successfulInitPhp = false;
            } else {
                $pos = strrpos($contentInitPhp, '?>');
                if ($pos === false) {
                    $successfulInitPhp = file_put_contents($pathInitPhp, PHP_EOL . $delobankInit, FILE_APPEND | LOCK_EX);
                    $error = 'Не выходит поправить init.php, возможно не хватает прав';
                } else {
                    $newContentInitPhp = substr($contentInitPhp, 0, $pos) . PHP_EOL .  $delobankInit;
                    $successfulInitPhp = file_put_contents($pathInitPhp, $newContentInitPhp, LOCK_EX);
                    $error = 'Не выходит поправить init.php, возможно не хватает прав';
                }
            }
        } else {
            $successfulInitPhp = file_put_contents($pathInitPhp, '<?php' . PHP_EOL . $delobankInit);
            $error = 'Не выходит создать init.php, возможно не хватает прав';
        }
        if ($successfulInitPhp) {
            return true;
        } else {
            $this->errors = $error;
        }
        return false;
    }

    public function UnInstallFiles()
    {
        $successfulInitPhp = false;
        $pathInitPhp = $this->getPathInitPhp();
        if (file_exists($pathInitPhp)) {
            $contentInitPhp = file_get_contents($pathInitPhp);
            $delobankInit = $this->includeCodeDelobankInitPhp();
            $pos = strrpos($contentInitPhp, $delobankInit);
            if ($pos !== false) {
                $lengthFindStr = mb_strlen($delobankInit);
                $newContentInitPhp = substr($contentInitPhp, 0, $pos) . "\n" . substr($contentInitPhp, $pos + $lengthFindStr, mb_strlen($contentInitPhp));
                $successfulInitPhp = file_put_contents($pathInitPhp, $newContentInitPhp, LOCK_EX);
            }
        }
        if ($successfulInitPhp) {
            DeleteDirFilesEx('/bitrix/modules/sale/payment/' . self::MODULE_ID);
            DeleteDirFilesEx('/personal/order/payment/' . self::MODULE_ID);
            DeleteDirFilesEx('/bitrix/modules/sale/payment/' . self::MODULE_ID . 'qrcode');
            DeleteDirFilesEx('/personal/order/payment/' . self::MODULE_ID . 'qrcode');
            DeleteDirFilesEx('/bitrix/php_interface/' . self::MODULE_ID . '_init.php');
            return true;
        }
        return false;
    }

    /**
     * Проверяте наличие файла init.php в директории local и если он есть то возвращает путь до него
     * @return string путь до файла init.php
     */
    private function getPathInitPhp()
    {
        $pathInitPhp = $_SERVER['DOCUMENT_ROOT'] . '/local/php_interface/init.php';
        if (file_exists($pathInitPhp)) {
            return $pathInitPhp;
        } else {
            return $_SERVER['DOCUMENT_ROOT'] . '/bitrix/php_interface/init.php';
        }
    }

    /**
     * Возвращает кусок кода с подключением нашего файла
     * @return string кусок кода с подключением нашего файла
     */
    private function includeCodeDelobankInitPhp()
    {
        return 'try {require __DIR__ . \'/' . self::MODULE_ID . '_init.php\';} catch (ParseError $e) {CEventLog::Add([\'SEVERITY\' => \'ERROR\', \'AUDIT_TYPE_ID\' => \'DELOBANK PAYMENT\', \'MODULE_ID\' => \'main\', \'ITEM_ID\' => 123, \'DESCRIPTION\' => $e->getMessage()]);}';
    }

    /**
     * Пишет в журнал событий
     * @param string $message сообщение
     * @param string $type может принимать значения SECURITY, ERROR, WARNING, INFO, DEBUG
     * @param int $itemId id пользователя или другого объекта
     */
    private function inLog($message, $type = 'ERROR', $itemId = 123)
    {
        CEventLog::Add([
            'SEVERITY'      => $type,
            'AUDIT_TYPE_ID' => 'DELOBANK PAYMENT',
            'MODULE_ID'     => 'main',
            'ITEM_ID'       => $itemId,
            'DESCRIPTION'   => $message,
        ]);
    }

    /**
     * Необходимые требования
     * @return bool
     */
    private function validateRequirements()
    {
        $minPhpVersion = 7.0;
        if (!version_compare(PHP_VERSION, $minPhpVersion, '>=')) {
            $this->errors = 'На вашем сервере до сих пор использую PHP ' . PHP_VERSION . ', для модуля "'
                . $this->MODULE_NAME . ' v' . $this->MODULE_VERSION . '" необходима минимум версия: ' .  $minPhpVersion;
            return false;
        }
        if (!extension_loaded('openssl')) {
            $this->errors = 'В PHP должно быть установлено расширение openSSL';
            return false;
        }
        return true;
    }
}
