<?php
namespace Delo;

require_once $_SERVER['DOCUMENT_ROOT'] . '/bitrix/modules/sale/lib/eventactions.php';
require_once $_SERVER['DOCUMENT_ROOT'] . '/bitrix/modules/sale/payment/delobank/DeloProcessor.php';

class DeloBackend
{
    /** @var int время проверки в часах */
    const CHECK_TIME = 48;

    /**
     * Будет вызыватся каждый раз при заходе в список заказов в админке и проверять результат платежа по QR-коду
     * @param \Bitrix\Main\Event $getListParams параметры выбора заказов
     * @throws \Bitrix\Main\ArgumentNullException
     * @throws \Bitrix\Main\ArgumentOutOfRangeException
     */
    public static function checkPaymentMulti($getListParams)
    {
        $orders = self::getOrders();
        if (empty($orders)) {
            return ;
        }
        $exampleOrder = current($orders);
        $paymentSettings = \DeloProcessor::getPaymentSettings($exampleOrder['PAY_SYSTEM_ID'], $exampleOrder['PERSON_TYPE_ID'], 2);
        $qrcodes = array_keys($orders);
        $result = \DeloProcessor::queryGetPaymentStatus(
            $paymentSettings['URL_QRCODE'],
            $paymentSettings['LOGIN_QRCODE'],
            $paymentSettings['PASSWORD_QRCODE'],
            '00000000-0000-0000-0000-000000000000',
            $qrcodes
        );
        if (empty($result['payments'])) {
            return ;
        }
        $updateArr = [];
        foreach ($result['payments'] as $payment) {
            if (($payment['status'] == \DeloProcessor::QRCODE_STATUS_SUCCESS) || ($payment['status'] == \DeloProcessor::QRCODE_STATUS_REJECTED)) {
                $updateArr[$payment['status']][] = [
                    'qrc_id'        => $payment['qrcId'],
                    'qrc_extension' => $orders[$payment['qrcId']]['qrc_extension'],
                    'order_id'      => $orders[$payment['qrcId']]['order_id'],
                ];
                if ($payment['status'] == \DeloProcessor::QRCODE_STATUS_SUCCESS) {
                    \DeloProcessor::setStatusCode(
                        $orders[$payment['qrcId']]['order_id'],
                        1,
                        $orders[$payment['qrcId']]['PRICE'],
                        '',
                        $paymentSettings,
                        ''
                    );
                }
            }
        }
        if (!empty($updateArr)) {
            self::updateQRCodeDataStatus($updateArr);
        }
        return ;
    }

    /**
     * Возвращает заказы, которые необходимо проверить
     * @return array
     */
    private static function getOrders()
    {
        global $DB;
        $time = time() - (self::CHECK_TIME * 60 * 60);
        $q = 'SELECT
                `qrcode`.`order_id`,
                `qrcode`.`qrc_id`,
                `qrcode`.`qrc_extension`,
                `orders`.`PAY_SYSTEM_ID`,
                `orders`.`PRICE`,
                `orders`.`PERSON_TYPE_ID`
            FROM `b_delobank_order_qrcode` AS `qrcode`
                LEFT JOIN `b_sale_order` AS `orders` ON `orders`.`ID` = `qrcode`.`order_id`
            WHERE `qrcode`.`date_create` > ' . $time . ' AND `qrcode`.`status` = "' . \DeloProcessor::QRCODE_STATUS_DEFAULT.  '"';
        $r = $DB->Query($q);
        $orders = [];
        while ($row = $r->Fetch()) {
            if (!empty($row['qrc_id'])) {
                $orders[$row['qrc_id']] = $row;
            }
        }
        return $orders;
    }

    /**
     * Обновляет информацию об оплатах по QR-кодам
     * @param array $updateArr
     */
    private static function updateQRCodeDataStatus($updateArr)
    {
        global $DB;
        if (empty($updateArr)) {
            return ;
        }
        foreach ($updateArr as $status => $dataQRCodes) {
            $ids = [];
            foreach ($dataQRCodes as $dataQRCode) {
                $ids[] = $dataQRCode['order_id'];
                unlink($_SERVER['DOCUMENT_ROOT'] . '/personal/order/payment/delobankqrcode/img/' . $dataQRCode['qrc_id'] . '.' . $dataQRCode['qrc_extension']);
            }
            if (!empty($ids)) {
                $q = 'UPDATE `b_delobank_order_qrcode` SET
                        `status` = "' . $status . '"
                    WHERE `order_id` IN (' . implode(',', $ids) . ')';
                $DB->Query($q);
            }
        }
    }

    /**
     * Возвращает uid QR-кода по id заказа
     * @param int $orderId
     * @return array|null
     */
    private static function getQRCodeUidByOrderId($orderId)
    {
        global $DB;
        $q = 'SELECT
                `qrc_id`,
                `qrc_extension`
            FROM `b_delobank_order_qrcode`
            WHERE `order_id` = ' . $orderId;
        $r = $DB->Query($q);
        $dataQRCode = $r->Fetch();
        if (!empty($dataQRCode['qrc_id'])) {
            return $dataQRCode;
        }
        return null;
    }

    /**
     * @param array $arg
     * @return array|void
     * @throws \Bitrix\Main\ArgumentNullException
     * @throws \Bitrix\Main\ArgumentOutOfRangeException
     */
    public static function buttonCheckPayment($arg)
    {
        $orderId = (int)$arg['ORDER']->getId();
        if (empty($orderId)) {
            return ;
        }
        if (!empty($_GET['delobankqrcode_check_payment'])) {
            self::checkPayment($orderId, $arg['ORDER']->getPrice(), current($arg['ORDER']->getPaySystemIdList()), $arg['ORDER']->getPersonTypeId());
        }
        $qrcodeData = self::getQRCodeUidByOrderId($orderId);
        if (empty($qrcodeData)) {
            return ;
        }
        return [
            'BLOCKSET'        => 'DeloBank QR Code Payments',
            "getScripts"      => ['Delo\\DeloBackend', 'mygetScripts'],
            "getBlocksBrief"  => ['Delo\\DeloBackend', 'mygetBlocksBrief'],
            "getBlockContent" => ['Delo\\DeloBackend', 'mygetBlockContent'],
        ];

    }

    public static function mygetBlocksBrief($args)
    {
        return [
            'delobankqrcode' => ['TITLE' => 'Проверка оплаты по QR-коду'],
        ];
    }

    public static function mygetScripts($args)
    {
        $url = $_SERVER['REQUEST_URI'] . '&delobankqrcode_check_payment=1';
        return '<script>
            function checkPayment() {
                var XHR = (\'onload\' in new XMLHttpRequest()) ? XMLHttpRequest : XDomainRequest;
                var xhr = new XHR();
                xhr.open(\'POST\', \'' . $url . '\', true);
                xhr.onloadstart = function () {
                    delobankqrcode_check_payment_loading.style.display = \'block\';
                };
                xhr.onload = function () {
                    delobankqrcode_check_payment_loading.style.display = \'none\';
                    if (xhr.status === 200) {
                        document.location.reload(true);
                    }
                };
                xhr.send(null);
            }
        </script>';
    }

    public static function mygetBlockContent($blockCode, $selectedTab, $args)
    {
        $result = '';
        if ($selectedTab == 'tab_order') {
            if ($blockCode == 'delobankqrcode') {
                $result = '<span id="delobankqrcode_check_payment" class="adm-btn" onclick="checkPayment()">Проверить оплату</span>
<div style="display:none;position:fixed;width:100%;height:100%;top:0;left:0;z-index:100;background:rgba(255,255,255,.6) url(/bitrix/components/bitrix/sale.order.ajax/templates/.default/images/loader.gif) center center no-repeat" id="delobankqrcode_check_payment_loading"></div>';
            }
        }

        return $result;
    }

    /**
     * @param int $orderId
     * @param float $orderPrice
     * @param int $paymentId
     * @param int $personId
     * @throws \Bitrix\Main\ArgumentNullException
     * @throws \Bitrix\Main\ArgumentOutOfRangeException
     */
    private static function checkPayment($orderId, $orderPrice, $paymentId, $personId)
    {
        $dataQRCode = self::getQRCodeUidByOrderId($orderId);
        $paymentSettings = \DeloProcessor::getPaymentSettings($paymentId, $personId, 2);
        $result = \DeloProcessor::queryGetPaymentStatus(
            $paymentSettings['URL_QRCODE'],
            $paymentSettings['LOGIN_QRCODE'],
            $paymentSettings['PASSWORD_QRCODE'],
            '00000000-0000-0000-0000-000000000000',
            [$dataQRCode['qrc_id']]
        );
        if (empty($result['payments'])) {
            return ;
        }
        $updateArr = [];
        foreach ($result['payments'] as $payment) {
            if (($payment['status'] == \DeloProcessor::QRCODE_STATUS_SUCCESS) || ($payment['status'] == \DeloProcessor::QRCODE_STATUS_REJECTED)) {
                $updateArr[$payment['status']][] = [
                    'qrc_id'        => $dataQRCode['qrc_id'],
                    'qrc_extension' => $dataQRCode['qrc_extension'],
                    'order_id'      => $orderId,
                ];

                if ($payment['status'] == \DeloProcessor::QRCODE_STATUS_SUCCESS) {
                    \DeloProcessor::setStatusCode($orderId, 1, $orderPrice, '', $paymentSettings, '');
                }
            }
        }
        if (!empty($updateArr)) {
            self::updateQRCodeDataStatus($updateArr);
        }
        return ;
    }
}

\Bitrix\Main\EventManager::getInstance()->addEventHandler('sale', \Bitrix\Sale\EventActions::EVENT_ON_ADMIN_ORDER_LIST, ['Delo\\DeloBackend', 'checkPaymentMulti']);
\Bitrix\Main\EventManager::getInstance()->addEventHandler("main", "OnAdminSaleOrderViewDraggable", ['Delo\\DeloBackend', 'buttonCheckPayment']);
\Bitrix\Main\EventManager::getInstance()->addEventHandler("main", "OnAdminSaleOrderEditDraggable", ['Delo\\DeloBackend', 'buttonCheckPayment']);